#
# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
#
from aistore.sdk.errors import InvalidObjectRangeIndex


# pylint: disable=unused-variable
class ObjectRange:
    def __init__(
        self,
        prefix: str,
        min_index: int,
        max_index: int,
        pad_width: int = 0,
        step: int = 1,
        suffix: str = "",
    ):
        """
        Args:
            prefix (str): Prefix contained in all names of objects
            min_index (int): Starting index in the name of objects
            max_index (int): Last index in the name of all objects
            pad_width (int): Left-pad indices with zeros up to the width provided, e.g. pad_width = 3 will transform 1
                to 001
            step (int, optional): Size of iterator steps between each item
            suffix (str, optional): Suffix at the end of all object names
        """

        self._prefix = prefix
        self._step = step
        self._suffix = suffix
        self._validate_indices(min_index, max_index, pad_width)
        self._min_index = str(min_index).zfill(pad_width)
        self._max_index = str(max_index).zfill(pad_width)

    @staticmethod
    def _validate_indices(min_index, max_index, pad_width):
        """
        Validate the indices passed to create a range: min_index < max_index and pad_width (if set) can fit the indices
            provided.

        Raises:
            InvalidObjectRangeIndex: If the indices passed to the range are not valid
        """
        if min_index >= max_index:
            raise InvalidObjectRangeIndex(f"{min_index} must be less than {max_index}")
        if pad_width != 0 and len(str(min_index)) > pad_width:
            raise InvalidObjectRangeIndex(
                f"Number of digits in min index {min_index} must not be greater than pad width {pad_width}"
            )

    def __str__(self) -> str:
        return f"{self._prefix}{{{self._min_index}..{self._max_index}..{self._step}}}{self._suffix}"



from aistore.sdk.api import Client

# If there is a version module generated by the setup script, import the version info into this module
# Otherwise ignore so the sdk works locally without running setup
try:
    from aistore.version import __version__
except ImportError:
    pass



#
# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
#
from aistore.sdk.const import (
    HTTP_METHOD_DELETE,
    ACT_DELETE_MULTIPLE_OBJ,
    ACT_PREFETCH_MULTIPLE_OBJ,
    HTTP_METHOD_POST,
    ACT_EVICT_MULTIPLE_OBJ,
)
from aistore.sdk.object_range import ObjectRange


# pylint: disable=unused-variable
class ObjectGroup:
    """
    A class representing multiple objects within the same bucket. Only one of obj_names or obj_range should be provided.

    Args:
        bck (Bucket): Bucket the objects belong to
        obj_names (list[str], optional): List of object names to include in this collection
        obj_range (ObjectRange, optional): Range defining which object names in the bucket should be included
        obj_template (str, optional): String argument to pass as template value directly to api
    """

    def __init__(
        self,
        bck,
        obj_names: list = None,
        obj_range: ObjectRange = None,
        obj_template: str = None,
    ):
        self.bck = bck
        num_args = sum(
            1 if x is not None else 0 for x in [obj_names, obj_range, obj_template]
        )
        if num_args != 1:
            raise ValueError(
                "ObjectGroup accepts one and only one of: obj_names, obj_range, or obj_template"
            )
        self.obj_names = obj_names
        self.obj_range = obj_range
        self.obj_template = obj_template

    def delete(self):
        """
        Deletes a list or range of objects in a bucket

        Raises:
            aistore.sdk.errors.AISError: All other types of errors with AIStore
            requests.ConnectionError: Connection error
            requests.ConnectionTimeout: Timed out connecting to AIStore
            requests.exceptions.HTTPError: Service unavailable
            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
            requests.ReadTimeout: Timed out receiving response from AIStore

        Returns:
            Job ID (as str) that can be used to check the status of the operation

        """

        return self.bck.make_request(
            HTTP_METHOD_DELETE, ACT_DELETE_MULTIPLE_OBJ, value=self._determine_value()
        ).text

    def evict(self):
        """
        Evicts a list or range of objects in a bucket so that they are no longer cached in AIS
        NOTE: only Cloud buckets can be evicted.

        Raises:
            aistore.sdk.errors.AISError: All other types of errors with AIStore
            requests.ConnectionError: Connection error
            requests.ConnectionTimeout: Timed out connecting to AIStore
            requests.exceptions.HTTPError: Service unavailable
            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
            requests.ReadTimeout: Timed out receiving response from AIStore

        Returns:
            Job ID (as str) that can be used to check the status of the operation

        """
        self.bck.verify_cloud_bucket()
        return self.bck.make_request(
            HTTP_METHOD_DELETE, ACT_EVICT_MULTIPLE_OBJ, value=self._determine_value()
        ).text

    def prefetch(self):
        """
        Prefetches a list or range of objects in a bucket so that they are cached in AIS
        NOTE: only Cloud buckets can be prefetched.

        Raises:
            aistore.sdk.errors.AISError: All other types of errors with AIStore
            requests.ConnectionError: Connection error
            requests.ConnectionTimeout: Timed out connecting to AIStore
            requests.exceptions.HTTPError: Service unavailable
            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
            requests.ReadTimeout: Timed out receiving response from AIStore

        Returns:
            Job ID (as str) that can be used to check the status of the operation

        """
        self.bck.verify_cloud_bucket()
        return self.bck.make_request(
            HTTP_METHOD_POST, ACT_PREFETCH_MULTIPLE_OBJ, value=self._determine_value()
        ).text

    def _determine_value(self):
        if self.obj_names:
            return {"objnames": self.obj_names}
        if self.obj_range:
            return {"template": str(self.obj_range)}
        if self.obj_template:
            return {"template": self.obj_template}
        return None



#
# Copyright (c) 2022-2023, NVIDIA CORPORATION. All rights reserved.
#

from __future__ import annotations  # pylint: disable=unused-variable
from typing import List
import time

from aistore.sdk.const import HTTP_METHOD_GET, HTTP_METHOD_PUT, QParamWhat, QParamForce
from aistore.sdk.errors import Timeout
from aistore.sdk.types import Bck, JobStatus, JobArgs
from aistore.sdk.utils import probing_frequency


# pylint: disable=unused-variable
class Job:
    """
    A class containing job-related functions.

    Args:
        None
    """

    def __init__(self, client):
        self._client = client

    @property
    def client(self):
        """The client bound to this job object."""
        return self._client

    def status(
        self,
        job_id: str = "",
        job_kind: str = "",
        daemon_id: str = "",
        only_running: bool = False,
    ) -> JobStatus:
        """
        Return status of a job

        Args:
            job_id (str, optional): UUID of the job. Empty - all jobs.
            job_kind (str, optional): Kind of the job. Empty - all kinds.
            daemon_id (str, optional): Return jobs only running on the daemon_id.
            only_running (bool, optional):
                True - return only currently running jobs
                False - include finished and aborted jobs

        Returns:
            The job description.

        Raises:
            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
            requests.ConnectionError: Connection error
            requests.ConnectionTimeout: Timed out connecting to AIStore
            requests.ReadTimeout: Timed out waiting response from AIStore
        """
        return self.client.request_deserialize(
            HTTP_METHOD_GET,
            path="cluster",
            res_model=JobStatus,
            json=JobArgs(
                id=job_id, kind=job_kind, only_running=only_running, daemon_id=daemon_id
            ).get_json(),
            params={QParamWhat: "status"},
        )

    def wait_for_job(
        self,
        job_id: str = "",
        job_kind: str = "",
        daemon_id: str = "",
        timeout: int = 300,
    ):
        """
        Wait for a job to finish

        Args:
            job_id (str, optional): UUID of the job. Empty - all jobs.
            job_kind (str, optional): Kind of the job. Empty - all kinds.
            daemon_id (str, optional): Return jobs only running on the daemon_id.
            timeout (int, optional): The maximum time to wait for the job, in seconds. Default timeout is 5 minutes.

        Returns:
            None

        Raises:
            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
            requests.ConnectionError: Connection error
            requests.ConnectionTimeout: Timed out connecting to AIStore
            requests.ReadTimeout: Timed out waiting response from AIStore
            errors.Timeout: Timeout while waiting for the job to finish
        """
        passed = 0
        sleep_time = probing_frequency(timeout)
        while True:
            if passed > timeout:
                raise Timeout("wait for job to finish")
            status = self.status(job_id=job_id, job_kind=job_kind, daemon_id=daemon_id)
            if status.end_time != 0:
                break
            time.sleep(sleep_time)
            passed += sleep_time
            print(status)

    def start(
        self,
        job_kind: str = "",
        daemon_id: str = "",
        force: bool = False,
        buckets: List[Bck] = None,
    ) -> str:
        """
        Start a job and return its ID.

        Args:
            job_kind (str, optional): Kind of the job (for supported kinds, see api/apc/const.go). Empty - all kinds.
            daemon_id (str, optional): Return jobs only running on the daemon_id.
            force (bool, optional): Override existing restrictions for a bucket (e.g., run LRU eviction even if the bucket has LRU disabled).
            buckets (List[Bck], optional): List of one or more buckets; applicable only for jobs that have bucket scope (for details and full enumeration, see xact/table.go).

        Returns:
            The running job ID.

        Raises:
            requests.RequestException: "There was an ambiguous exception that occurred while handling..."
            requests.ConnectionError: Connection error
            requests.ConnectionTimeout: Timed out connecting to AIStore
            requests.ReadTimeout: Timed out waiting response from AIStore
        """
        value = JobArgs(kind=job_kind, daemon_id=daemon_id, buckets=buckets).get_json()
        params = {QParamForce: "true"} if force else {}
        action = {"action": "start", "value": value}

        resp = self.client.request(
            HTTP_METHOD_PUT, path="cluster", json=action, params=params
        )
        return resp.text
